#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""University of New Hampshire data format

Copyright (C) 2014-2023 Christian T. Steigies <steigies@physik.uni-kiel.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

This module offers functions to read NM data in ASCII from an FTP server
as used in New Hampshire and return data in the format as used by NMDB."""

__author__ = ["Malcolm Colson <malcolm.colson@unh.edu>",
              "Andrew Kuhlman <andrew.kuhlman@unh.edu>",
              "Christian T. Steigies <steigies@physik.uni-kiel.de>"]
__license__ = "GPL License"

import pandas as pd
import sqlalchemy
import sys

TABLE_NAMES = { "drhm": "Durham",
                "mtws": "MWO",
                "ldvl": "Leadville",
                "hela": "HLEA",
                "clmx": "Climax",
                "huan": "Huancayo"
              }

# --------------------------------------------------------------------------
def get_sql_info(info_file):
    file = open(info_file, 'r')
    lines = file.readlines()

    user = lines[0].rstrip()
    password = lines[1].rstrip()
    
    return user, password
# --------------------------------------------------------------------------
def read_rt(station, data_type, year, month, day):
    """read in data from UNH/Durham"""
    try:
        sql_user, sql_pass = get_sql_info('/Users/Malcolm/Desktop/NMDB/.psqlinfo')
        conn_string = f'postgresql://{sql_user}:{sql_pass}@localhost/Neutron Monitor'
        db = sqlalchemy.create_engine(conn_string)

        start_date = f'{year}-{month}-{day}'
        query = (f"""SELECT * 
                     FROM public."{TABLE_NAMES[station]}"
                     WHERE DATE("Timestamp") = '{start_date}';""")

        df = pd.read_sql(query, con=db)
        if df.empty is True:
            print("no DATA")
            return None

        df.rename(columns={'Timestamp': 'start_date_time',
                           'Total': 'uncorr',
                           'Corr_Total': 'corr',
                           'HP_Barometer': 'press'}, inplace=True)

        if data_type in ["1m", "ori"]:
            df['interval'] = 60.
        elif data_type in ["1h"]:
            data = data.resample('1hr', on='Timestamp').mean()
            df['interval'] = 3600.
        else:
            raise ValueError  # unknown data_type in read_pandas

        df['ceff'] = df['corr']

        return df
    except ValueError:
        return None


# --------------------------------------------------------------------------
def read_rev(station, data_type, year, month, day):
    return read_rt(station, data_type, year, month, day)

# --------------------------------------------------------------------------
if __name__ == '__main__':
    station = "drhm"
    data_type="1m"
    year = sys.argv[1]
    month = sys.argv[2]
    day = sys.argv[3]

    df = read_rev(station, data_type, year, month, day)
    df.sort_values(by = 'start_date_time', inplace = True)
    print(df.head())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""KIEL0 (IGY and NM64 before IZMIRAN) data format

Copyright (c) 2009-2024 Christian T. Steigies <steigies@physik.uni-kiel.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

This module offers functions to read NM data in ASCII format as used in KIEL.

KIEL data availability
1h data since:
1957-07-01 C and O data only, no U or P data
1964-09-01 C, O, U and P data

1min data since:
TODO (early data is actually in 5min resolution?)
1971-02-01 C, O and P data
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"

import pandas as pd

my_station = ["kiel0"]
my_names = {"kiel0": "KIEL0"}
my_station_longnames = {"kiel0": "Kiel (IGY and NM64)"}
pressure = {"kiel0": [900., 1200.]}
countrate = {"kiel0": [10., 1000.]}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jul 26 10:33:02 2023

@author: steigies
"""
from datetime import datetime, date, time
import pandas as pd
import numpy as np
from nmdb.station.station_name import station_name

# --------------------------------------------------------------------------
def idate(year, month, day):
    timestamp = date(int(year), int(month), int(day))
    return timestamp

# --------------------------------------------------------------------------
def itime(hour, minute, sec):
    timestamp = time(int(hour), int(minute), int(sec))
    return timestamp


# --------------------------------------------------------------------------
def last_datapoint(cursor, table, station):
    """return timestamp of last value for station in real-time table"""

    if table in ["1m"]:  # 1m real-time data is in ori table
        table = "ori"
    elif table in ["ori", "1h", "env", "meta"]:
        pass
    elif table in ["rev"]:
        raise ValueError  # rev is not a real-time table
    else:
        raise ValueError  # illegal table

    station = station_name(station)  # kiel3 is stored as kiel2
    #print("last_datapoint for %s" %(station))
    select = "SELECT start_date_time FROM"
    order = "ORDER BY start_date_time DESC LIMIT 1"
    querystring = "%s %s_%s %s;" % (select, station.upper(), table, order)
    cursor.execute(querystring)
    answer = cursor.fetchone()
    if answer is None:
        print("%s_%s is empty, you should select a --startdate. "
        "Using default date." % (station.upper(), table))
        lastdatapoint = datetime(2000, 1, 1, 0, 0, 0)
    else:
        lastdatapoint = answer[0]

    #return(lastdatapoint)
    return(pd.to_datetime(lastdatapoint))


# --------------------------------------------------------------------------
def nmdb_timestamp(cursor):
    """get current timestamp from database server"""

    qstring = "SELECT CURRENT_TIMESTAMP()"
    cursor.execute(qstring)
    value = cursor.fetchone()

    return value[0]


# --------------------------------------------------------------------------
def str2datetime(string, zero=False):
    """convert string into datetime timestamp"""

    if string:
        # datetime.strptime is not available on CentOS
        # dt = datetime.strptime(s, "%Y-%m-%d %H:%M:%S")

        if "T" in string:
            (mydate, mytime) = string.split("T")
            # print(mydate)
            if "Z" in mytime:
                mytime = mytime[:-1]
                # print(mytime)
        else:
            (mydate, mytime) = string.split()
        (year, month, day) = mydate.split("-")
        (hour, minute, sec) = mytime.split(":")
        if zero:  # set seconds to zero
            sec = 0
        # timestamp = datetime(int(year), int(month), int(day), int(hour), int(minute), int(sec))
        timestamp = datetime.combine(idate(year, month, day),
                                     itime(hour, minute, sec))
        #return timestamp
        return np.datetime64(timestamp)
    else:
        raise ValueError


# --------------------------------------------------------------------------
def sys_timestamp(format="dt"):
    """get system time in UTC"""
    from time import gmtime, strftime
    now = gmtime()
    if format in ["dt", "datetime"]:
        value = datetime(now.tm_year, now.tm_mon, now.tm_mday, now.tm_hour, now.tm_min, now.tm_sec)
    elif format in ["str", "string"]:
        value = strftime("%Y-%m-%d %H:%M:%S", gmtime())
    else:
        raise ValueError  # unknown format
    return value





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jul 13 14:10:20 2023

@author: steigies
"""

import argparse  # https://docs.python.org/3/library/argparse.html
# https://docs.python.org/3/howto/argparse.html#argparse-tutorial

# __version__ = VERSION
__version__ = "0.1.7"
# BUG  nmdb --version prints 0.1.3

# --------------------------------------------------------------------------
def nmdb_parser():
    parser = argparse.ArgumentParser(description='NMDB commandline interface, public version.')
    parser.add_argument('--version', action='version', version='This is nmdb '+ __version__)
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-v', '--verbose', action='count', default=0, help='increase verbosity level')
    group.add_argument("-q", "--quiet", action="store_true")
    parser.set_defaults(cmd=None)
    
    # add subparsers!
    subparsers = parser.add_subparsers() #help='nmdb sub-commands')

    from nmdb.status import parser as parser_status
    parser_status = parser_status(subparsers)
    from nmdb.realtime import parser as parser_realtime
    parser_realtime = parser_realtime(subparsers)

    # create the parser for the "revise" command
    parser_rev = subparsers.add_parser('revise', help='revise NMDB data')
    parser_rev.add_argument('station', metavar='STATION', nargs='?', default='test', help='NMDB station short name')
    parser_rev.add_argument('data_type', choices=['1m', '1h', 'env', 'meta'], nargs='?', default='1m')
    parser_rev.add_argument('--dry-run', dest='dryrun', action="store_true", help='print but do not execute commands')
    parser_rev.add_argument('--file', dest='rcfile', help='config file for NMDB')
    parser_rev.add_argument('-H', '--host', dest='hostname', help='NMDB hostname to connect to')
    parser_rev.add_argument('-P', '--port', type=int, dest='port', default='3306', help='NMDB port number to connect to')
    parser_rev.add_argument('-Y', '--year', type=int, dest='year', help='year')
    parser_rev.add_argument('-M', '--month', type=int, dest='month', help='month')
    parser_rev.add_argument('-D', '--day', type=int, dest='day', help='day')
    group = parser_rev.add_mutually_exclusive_group()
    group.add_argument('-v', '--verbose', action='count', default=0, help='increase verbosity level')
    group.add_argument("-q", "--quiet", action="store_true")
    parser_rev.set_defaults(cmd="revise")

    from nmdb.upload import parser_upload
    parser_upload = parser_upload(subparsers)
    from nmdb.download import parser_download
    parser_download = parser_download(subparsers)

    return(parser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jul 25 12:11:01 2023

@author: steigies
"""

my_station = ["kiel"]
pressure = [900., 1200.]
countrate = [10., 1000.]

def read_rt(args):
    if args.station in my_station:
        print(args.station)
    else:
        raise ValueError  # wrong station

    print("read_rt KIEL")


    return None



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""mysql helper functions

Copyright (C) 2012-2019 Christian T. Steigies <steigies@physik.uni-kiel.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"

from datetime import datetime
#from nmdb3.nmdb.mysql_query import query, query_1h
from nmdb.mysql.nmdb_format import hour_format, ori_format, rev_format, meta_format, env_format
from nmdb.station.station_name import station_name

## --------------------------------------------------------------------------
def query(name, version, mode):
    if mode in ["ori", "1m"]:
        value = query_1m(name, version)
    elif mode in ["1h"]:
        value = query_1h(name, version)
    else:
        raise ValueError  # illegal mode

    return value
## --------------------------------------------------------------------------

def query_1h(name, version):
    """
    create write query for 1h data
    use REPLACE only when version == 1
    TODO rename to insert_1h?
    """

    station = station_name(name)  # kiel3 is stored as kiel2
    # print("writing to table: "+ station)

    if version == 0:  # initial write
        value = ("INSERT INTO %s_1h (%s)") % (station.upper(), hour_format)

    elif version == 1:  # overwrite
        value = ("REPLACE INTO %s_1h (%s)") % (station.upper(), hour_format)

    else:
        print(station, version)
        raise ValueError  # illegal version number

    return value


## --------------------------------------------------------------------------
def query_1m(name, version):
    """
    create write query for ori, rev, based on version number
    TODO rename to insert_revori?
    """

    station = station_name(name)  # kiel3 is stored as kiel2
    # print("writing to table: "+ station)

    if version == 0:  # version = "ori"
        value = ("INSERT INTO %s_ori (%s)") % (station.upper(), ori_format)

    elif  version == 1:  # rev version 1
        value = ("INSERT INTO %s_rev (%s)") % (station.upper(), rev_format)

    elif version > 1:  # rev version 2 or higher
        value = ("REPLACE INTO %s_rev (%s)") % (station.upper(), rev_format)

    else:
        raise ValueError  # illegal version number

    return value



## --------------------------------------------------------------------------
def write_nmdb(options, cursor, write):
    """write to NMDB"""

    value = 1

    if options.dryrun:  # dryrun counts as if data was written
        print(write)
    else:  # write data
        if options.verbose > 0:
            print(write)
        cursor.execute(write)
        answer = cursor.fetchone()
        if answer is not None:
            print(answer)

    return value


## --------------------------------------------------------------------------
def BUG_flush_nmdb(options, cursor, name, data_type, data, version):
    """INSERT or REPLACE 1h values"""

    # BUG mixup query and query_1h?
    if data_type in ["1m", "ori", "rev"]:
        write = query_1h(name, version)  # INSERT or REPLACE
    elif data_type in ["1h"]:
        write = query(name, version)  # INSERT or REPLACE, ori or rev
    else:
        raise ValueError  # wrong data_type

    if len(data) == 0:
        value = 0
    else:
        write += " VALUES " + mysql_values(data)
        value = write_nmdb(options, cursor, write)

    return value


## --------------------------------------------------------------------------
def mysql_values(data):
    """
    executemany uses many INSERTS with only one VALUE,
    instead of one INSERT with many VALUES
    see https://answers.launchpad.net/myconnpy/+question/120598
    until this is fixed, use execute instead of executemany
    and prepare the VALUES with this helper function
    """

    ret = ""
    delimiter = ""
    for row in data:
        ret += delimiter + mysql_row(row)
        if delimiter == "":
            delimiter = ", "

    return ret

## --------------------------------------------------------------------------
def mysql_row(row):
    """for one dataset, convert all values to strings and None to NULL"""

    ret = "("
    delimiter = ""
    for val in row:
        if val is None:
            val = "NULL"
        elif isinstance(val, str):
            val = "\"" + str(val) + "\""
        elif isinstance(val, datetime):
            val = "\"" + str(val) + "\""
        else:
            val = str(val)
        ret += delimiter + val
        if not delimiter:
            delimiter = ", "
    ret += ")"

    return ret

## --------------------------------------------------------------------------
if __name__ == "__main__":
    print("This is the mysql module.")

    data = [('2013-07-18 21:15:00', 60, 92.51, None, 90.68, 1010.519, 2), ('2013-07-18 21:17:00', 60, 93.37, None, 91.5, 1010.506, 2), ('2013-07-18 21:19:00', 60, 94.91, None, 93.02, 1010.519, 2), ('2013-07-18 21:22:00', 60, 91.99, None, 90.16, 1010.519, 2), ('2013-07-18 21:24:00', 60, 92.16, None, 90.3, 1010.479, 2), ('2013-07-18 21:25:00', 60, 92.47, None, 90.57, 1010.426, 2), ('2013-07-18 21:26:00', 60, 93.54, None, 91.61, 1010.413, 2), ('2013-07-18 21:27:00', 60, 92.38, None, 90.46, 1010.399, 2), ('2013-07-18 21:36:00', 60, 92.14, None, 90.22, 1010.386, 2), ('2013-07-18 21:38:00', 60, 97.11, None, 95.07, 1010.359, 2), ('2013-07-18 21:39:00', 60, 92.75, None, 90.79, 1010.346, 2), ('2013-07-18 21:40:00', 60, 95.68, None, 93.66, 1010.346, 2)]

    print(mysql_values(data))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
read Kiel3 data into a pandas dataframe in NMDB format
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"

import numpy as np
import pandas as pd
from datetime import datetime
#from statistics import median
import sqlite3
#from nmdb3.tools.datetool import str2datetime, valid_date

#from nmdb.tools.query import dt_start_stopp

# realtime: read nm_raw files
# revised: read from sqlite

my_station = ["kiel3"]
my_names = {"kiel3": "KIEL3"}
my_station_longnames = {"kiel3": "Kiel (nmrena)"}
# module may support reading data for multiple station, with different pressures
pressure = {"kiel0": [600., 1100.],  # (961.8 - 1041.05) mbar
            "kiel1": [700., 1100.],  # (961.8 - 1041.05) mbar
            "kiel2": [800., 1100.],  # (961.8 - 1041.05) mbar
            "kiel3": [900., 1100.],  # (961.8 - 1041.05) mbar
            }
countrate = {"kiel3": [60, 6000]}  # typical counts per tube (450-600) / min

TUBES = 18 # Kiel is an 18NM64

# one hour long-term averages
# taken from original IZMIRAN software
n0 = [#0, # first tube is ch_01 not 0
      30220, 35780, 36960, 37362, 37020, 31690,
      32550, 36785, 38000, 37284, 36826, 32270,
      32200, 37003, 37685, 37910, 37125, 31030]

# pressure sensors mapping to sqlite column names
#sensor= {"BM35": "p_bm35",
#         "BM35b": "p_bm35",
#         "int": "p_int",
#         "GB1": "p_gb1"}

# --------------------------------------------------------------------------
def min_value(station, measure, checked=False):
    """
    minimum value (countrate, pressure) the station will ever measure.
    if checked is True, accept 10 times lower values (for use in revise)
    """

    print(measure)
    if station in my_station:
        value = measure[station][0]
        if checked:
            value = value/10.
    else:
        raise ValueError # station not defined

    return (value)


# --------------------------------------------------------------------------
def max_value(station, measure, checked=False):
    """
    maximum value (countrate, pressure) the station will ever measure.
    if checked is True, accept 10 times higher values (for use in revise)
    """

    print(measure)
    if station in my_station:
        value = measure[station][1]
        if checked:
            value = value*10.
    else:
        raise ValueError # station not defined

    return (value)

# --------------------------------------------------------------------------
def corrected_for_pressure(uncorrected, pressure_mbar):
    """correct countrates to standard pressure"""
    from math import exp
    
    press_0 = 1006.7

    corrected = uncorrected * exp(0.00721*(pressure_mbar-press_0))

    return corrected

# --------------------------------------------------------------------------
def corrected_for_efficiency(corrected):
    """no efficiency correction for kiel (yet)"""

    return corrected

# --------------------------------------------------------------------------
def efficiencies(data):
    """efficiency: compare with (long term?) hour average of each channel"""
    eff = []
    for ch in range(0, TUBES):
        eff.append(data[ch]*60./n0[ch])
    
    return eff

# --------------------------------------------------------------------------
def tube2ch(station, num):
    """
    map tube number to data channel
    3 sections with 8 counters each
    the outer tubes of each section (0 and 7) are not connected
    """
    # TODO lists start at 0, median calculation is wrong if 0 is element 0
    # do not shift ch numbers by 1...
    # TODO map tubes to section Green, Yellow, Red?
    if station in "kiel":
        if num <= 0:
            raise ValueError  # tube number too small
        elif num <= TUBES/3:  # first section
            return num + 1
        elif num <= TUBES/3*2:  # second section
            return num + 3
        elif num <= TUBES:  # third section
            return num + 5
        else:
            raise ValueError  # tube number too big
    else:
        print("tube2ch not defined for station", station)
        raise ValueError

# --------------------------------------------------------------------------
def p_parser(press, key="BM35"):
#def p_parser(row, key="BM35"):
    """
    input: pressure as float or string as provided by NMRENA

    BM35 column may contain two values: BM35 (Mittlerer Luftdruck)
    and BM35b.(Momentaner Luftdruck). We use only BM35.
    find :BM35: in string. Numbers until next : are p_bm35
    since 2024-01-08: GB1 data is included in BM35 column, GB1 is empty!?!
    """

    #press = row[sensor[key]]
    #print("parser:", press, key)
    value = 0.  # no pressure, store as 0
    if isinstance(press, float):  # until 2022 a single BM35 value was used
        value = float(press)
    else:
        my_dict = {}
        if press[0] == ":":  # several key:value pairs are stored
            data = press.split(":")
            keyword = data.pop(0)  # pop off first ":"
            while len(data) > 1:  # need two values: keyword and value
                keyword = data.pop(0)
                value = data.pop(0)
                my_dict[keyword] = value

        try:
            value = my_dict[key]  # return pressure for the given key
        except KeyError:  # return 0 if key not found
            value = 0.

    return float(value)

# --------------------------------------------------------------------------
def median_editor(index, row):
    """
    Kiel median editor
    replace channels that deviate by more than 20% with median scaled values
    sum all (edited) channels for the total countrate
    perform pressure and efficiency corrections on sum
    ignore known bad channels
    ignore nan for median calculation: this changes the median!
    """
    p_min = pressure["kiel3"][0]
    p_max = pressure["kiel3"][1]
    cr_min = countrate["kiel3"][0]
    cr_max = countrate["kiel3"][1]

    ts = index.replace(second=0)  # set seconds to zero
    #print(row)
    length = float(row['integration'])
    # TODO check for length off by more than 1 sec?
    # TODO flag bad values, write to separate table for manual processing?
    press = p_parser(row['p_bm35'], 'BM35')  # BM35 averaged value
    # crude check for invalid data
    if (press < p_min) or (press > p_max):
        return None

    # TODO check for missing values, use other sensors. in read_rev?

    # first ch is ch01
    counts = np.zeros(TUBES)
    edited = np.zeros(TUBES)
    for i in range(0, TUBES):
        ch = "ch%02i" % (tube2ch('kiel', i+1))
        #if mode == "rt":
        #    ch = "ch%02i" % (tube2ch('kiel', i))
        #elif mode == "rev":
        #    ch = "ch_%02i" % (tube2ch('kiel', i))
        #else:
        #    raise ValueError  # wrong mode in median_editor
        counts[i] = int(row[ch])

    # eliminate bad channels: typical countrate per tube is 600
    # channels off by a factor of 10 are considered to be bad
    nobad = np.ma.masked_outside(counts, cr_min, cr_max)
    # convert mask to nan to prevent User Warning: converting a masked element to nan
    counts_nan = np.ma.filled(nobad.astype(float), np.nan)
    eff = efficiencies(counts_nan)  # eff without known bad channels
    m = np.nanmedian(eff)  # ignore NaN for median calculation

    edit_ch = 0
    for i in range(0, TUBES):
        # known bad channels or more than 20% deviation from median
        if nobad.mask[i] or (eff[i] < (m-0.2)) or (eff[i] > (m+0.2)):
            # take length of measurement (60s +- delta t) into account!?
            edited[i] = m*n0[i]/length  # replace by median scaled value
            edit_ch += 1
        else:
            edited[i] = counts[i]

    if False:
        print("counts", counts)
        print("nobad ", nobad)
        print("w. NaN", counts_nan)
        print("eff   ", eff)
        print("median", m)
        print("mask  ", nobad.mask)
        print("edited", edited)
        print()

    # TODO divide by length a second time?
    uncorr = sum(edited)/length
    corr = corrected_for_pressure(uncorr, press)
    ceff = corrected_for_efficiency(corr)

    df = pd.DataFrame({'start_date_time': [ts],
        'interval': [60],
        'uncorr': [uncorr],
        'ceff': [ceff],
        'corr': [corr],
        'press': [press],
        'edit_ch': [edit_ch],
        'bad_ch': [sum(nobad.mask)]})

    df.index = pd.to_datetime(df['start_date_time'])
    df.drop(["start_date_time"], axis=1, inplace=True)

    return(df)

# --------------------------------------------------------------------------
def read_rt(station, data_type, year, month, day):
    """
    read realtime data for one day at a time
    return dataframe in nmdb format
    """

    #print("read_rt: reading for", year, month, day)
    if data_type in  ["env", "meta"]:  # not yet implemented
        return None

    df_day = []  # list with dataframes for all hours of the day
    for hour in range(0, 24):
        #df_raw = []  # list with dataframes of raw data for one hour
        df = read_raw(year, month, day, hour)
        if df is not None:
            #df = pd.concat(df_raw)  # dataframe with Kiel3 raw data
            df_hour = []  # list of dataframes for one hour
            for index, row in df.iterrows():  # better: list comprehension...
                df_hour.append(median_editor(index, row))
            df_nmdb = pd.concat(df_hour)  # dataframe with Kiel3 data in NMDB format
            #print("df_nmdb", df_nmdb)
            if data_type in  ["1m"]:
                df_day.append(df_nmdb)
                #print(len(df_day))
            elif data_type in  ["1h"]:  # calculate one hour averages
                df_1h = pd.DataFrame({
                    'start_date_time': [pd.to_datetime(datetime(year, month, day, hour))],
                    'interval': [3600],
                    'uncorr': [df_nmdb.mean()["uncorr"]],
                    'ceff': [df_nmdb.mean()["ceff"]],
                    'corr': [df_nmdb.mean()["corr"]],
                    'press': [df_nmdb.mean()["press"]]})
                df_1h.index = pd.to_datetime(df_1h['start_date_time'])
                df_1h.drop(["start_date_time"], axis=1, inplace=True)
                df_day.append(df_1h)
        else:  # not yet defined, exit early
            return None
    #print(df_hour)  # check/warn number of bad channels
    return(pd.concat(df_day))  # dataframe with Kiel3 data in NMDB format

# --------------------------------------------------------------------------
def read_rev(station, data_type, year, month, day=None):
    """read revised data

    station     station short name, must be valid station
    data_type   "1m", "1h", "env", "meta"
    year        integer (1950-9999), may not be None

    month       integer (1-12), may not be None
    day         integer (1-31), if None, read day 1-31

    return dataframe in nmdb format

    read sqlitefile produced by lvl0, lvl1, lvl2, ... functions?
    """
    return None

# --------------------------------------------------------------------------
def read_lvl(station, lvl, year, month, day=None):
    """
    read data for lvl from sqlite
    """
    # TODO DRY! put this in a function, lvl as parameter
    if station in my_station:
        pass
    else:
        raise ValueError  # wrong station

    # sqlite file
    if lvl < 0:  # raw data, monthly files
        filename = "/data/nm64/kiel/%s_%04i-%02i.raw.sqlite" \
            % (station, year, month)
        table = 'data'
    else:  # all levels of revised data, yearly files
        filename = "/data/nm64/kiel3/%s_%04i.rev.sqlite" \
            % (station, year)
        table = 'lvl%i' %(lvl)
    try:
        con = sqlite3.connect(filename)
    except:
        return None

    (start, stopp) = dt_start_stopp(year, month, day)
    query = "SELECT * FROM '%s' WHERE datetime BETWEEN '%s' AND '%s'" %(table, start, stopp)
    #print(query)
    try:
        df = pd.read_sql_query(query, con)
    except Exception as error:  # OperationalError
        print(error)
        return(None)
    df.index = pd.to_datetime(df['datetime'])
    df.index = df.index.tz_localize(None)  # remove TZ
    df.drop(['datetime'], axis=1, inplace=True) # drop obsolete columns

    return(df, con)

# --------------------------------------------------------------------------
def lvl0(station, year, month, day=None):
    """
    create sqlite database with lvl0 data

    test case:
        "2023-10-17 17:13:00" (missing 17:12:00, double length?)
        "2022-07-20 09:30:00" BM35 contains only ":"
    """

    (df, con) = read_lvl(station, -1, year, month, day)
    con.close()
    #if station in my_station:
    #    pass
    #else:
    #    raise ValueError  # wrong station

    ## read from sqlite
    #filename = "/data/nm64/kiel/%s_%04i-%02i.raw.sqlite" \
    #        % (station, year, month)
    #try:
    #    con = sqlite3.connect(filename)
    #except:
    #    return None

    #(start, stopp) = dt_start_stopp(year, month, day)
    #query = "SELECT * FROM data WHERE datetime BETWEEN \"%s\" AND \"%s\"" % (start, stopp)
    #try:
    #    df = pd.read_sql_query(query, con)
    #    con.close()
    #except:  # OperationalError:  # raw.sqlite file does not exist
    #    return

    # use datetime as index
    #df.index = pd.to_datetime(df['datetime'])
    #df.drop(["datetime"], axis=1, inplace=True)  # drop unused columns

    # write to sqlite
    # TODO put everything in dataframe, df.to_sql
    filename = "/data/nm64/kiel3/%s_%04i.rev.sqlite" \
            % (station, year)
    try:
        con = sqlite3.connect(filename)
    except:
        return None

    cols = "datetime DATE PRIMARY KEY ASC, integration FLOAT, t_int FLOAT, p_int FLOAT, p_bm35 FLOAT, p_gb1 FLOAT"
    for i in range(1, TUBES+1):
        cols += ", ch%02i INT" % (i)
    query= "CREATE TABLE IF NOT EXISTS lvl0 (%s);" % (cols)
    con.execute(query)

    for index, row in df.iterrows():  # better: list comprehension...
        ts = index.replace(second=0)  # set seconds to zero
        length = float(row['integration'])
        t_int = row["t_int"]
        p_int = p_parser(row["p_int"], 'int')  # internal sensor
        p_bm35 = p_parser(row["p_bm35"], 'BM35')  # BM35 averaged value
        p_gb1 = p_parser(row["p_gb1"], 'GB1')  # GB1 not connected
        counts = np.zeros(TUBES+1)
        # copied from median_editor
        for i in range(1, TUBES+1):  # TODO channel 1-18 not 0-17!
            # TODO ch_01 or ch01?
            ch = "ch_%02i" % (tube2ch('kiel', i))
            #ch = "ch%02i" % (tube2ch('kiel', i))
            counts[i] = int(row[ch])

        data = "\"%s\", %f, %f, %f, %f, %f" % (ts, length, t_int, p_int, p_bm35, p_gb1)
        for i in range(1, TUBES+1):
            data += ", %i" % counts[i]
        query= "INSERT OR IGNORE INTO lvl0 VALUES (%s);" % (data)
        #print(query)
        con.execute(query)
    con.commit()
    con.close()

    return()

# --------------------------------------------------------------------------
def lvl1(station, year, month, day=None):
    """
    create lvl1 data
    find tubes with countrate out of range and set to NA
    find pressure out of range and set to NA
    """

    p_min = pressure["kiel3"][0]
    p_max = pressure["kiel3"][1]
    cr_min = countrate["kiel3"][0]
    cr_max = countrate["kiel3"][1]

    (df, con) = read_lvl(station, 0, year, month, day)
    # TODO rename column from interval to integration?

    # Verify that result of SQL query is stored in the dataframe
    #print(df.head())

    # BM35 barometer
    df.loc[df['p_bm35'] < p_min,'p_bm35'] = np.nan  # pressure too low
    df.loc[df['p_bm35'] > p_max,'p_bm35'] = np.nan  # pressure too high

    # internal barometer (less precision)
    df.loc[df['p_int'] < p_min,'p_int'] = np.nan  # pressure too low
    df.loc[df['p_int'] > p_max,'p_int'] = np.nan  # pressure too high

    # GB1 barometer (not connected)
    df.loc[df['p_gb1'] < p_min,'p_gb1'] = np.nan  # pressure too low
    df.loc[df['p_gb1'] > p_max,'p_gb1'] = np.nan  # pressure too high

    # this should be done in lvl2:
    # dataframe containing BM35 pressure measurements with errors:
    df_p = df.loc[pd.isna(df['p_bm35'])]
    #print(df_p)
    for idx, row in df_p.iterrows():
        press = interpolate_pressure(df, idx)
        df.at[idx, 'p_bm35'] = press  # replace missing value
        print("replacing p_bm35 at", idx, "from", row.p_bm35, "with", press)

    # eliminate counts that are clearly out of range for this station
    for i in range(1, TUBES+1):
        ch = "ch%02i" % (i)
        df.loc[df[ch] < cr_min,ch] = np.nan  # countrate too low
        df.loc[df[ch] > cr_max,ch] = np.nan  # countrate too high

    # TODO change dtypes of ch to INT instead of float64
    #df.to_sql(name='lvl1', if_exists='append', con=con)
    #df.to_sql(name='lvl1', if_exists='replace', con=con)
    con.close()

    return(df)

# --------------------------------------------------------------------------
def interpolate_pressure(df, ts):
    minutes = 2  # +- 2 minutes
    t_min = ts - pd.Timedelta(minutes=minutes)
    t_max = ts + pd.Timedelta(minutes=minutes)
    df2 = df[t_min:t_max][['p_int', 'p_bm35']]
    pressure = df2['p_bm35'].median()
    #print(pressure)

    return(pressure)

# --------------------------------------------------------------------------
def lvl2(station, year, month, day=None):
    """
    create lvl2 data
    replace tubes with countrate 0 by median scaled values
    select one pressure: p_bm35 preferred, if invalid: scale p_int
    data is invalid if interval deviates more than 1.2s from 60s
    """
    (df, con) = read_lvl(station, 1, year, month, day)

    return(df)
# --------------------------------------------------------------------------
def lvl3(station, year, month, day=None):
    """
    create lvl3 data
    sum all channels, pressure and efficiency correction
    """

# --------------------------------------------------------------------------
def read_raw(year, month, day, hour):
    """
    read raw NMIRENA data from file
    return all data in a pandas dataframe
    1       Uhrzeit UTC (des letzten HK pakets in der Minute)
    2       Uhrzeit in Sekunden
    3       Integrationszeit in Sekunden
    4       Temperatur des internen Drucksensors
    5       Luftdruck des internen Drucksensors
    6       Luftdruck des BM35 (seit 2024-01-08: auch GB1 statt BM35b?)
    7       Luftdruck des GB1 (seit 2024-01-08: ":" auf nm64data)
    8 - 15  ADC readout (future use: bias Voltages and Currents)
    16 - 39 Röhrenzählrate [min¯¹] (i.e., NICHT MEHR korrigiert nach Spalte 2)
    40 - 55 delta t-Spektrum [min¯¹] (i.e., NICHT MEHR korrigiert nach Spalte 2)
    """

    names = ["timestamp", "unixtime", "integration", "t_int",
             "p_int", "p_bm35", "p_gb1"]
    for i in range(8, 15+1):
        names.append("adc%02i" %(i-8))
    for i in range(16, 39+1):
        # BUG ?
        #names.append("ch%02i" %(i-16))  # ch00-17?
        names.append("ch%02i" %(i-15))  # ch01-18
    for i in range(40, 55+1):
        names.append("dt%02i" %(i-40))

    if year < 2023:
        filename = "~/data/kiel/%04i/%02i/kiel_%04i-%02i-%02iT%02iZ.nm_raw" \
            % (year, month, year, month, day, hour)
    else:
        #filename = "~/data/kiel/kiel_%04i-%02i-%02iT%02iZ.nm_raw" \
        filename = "/data/falbala/nm64/data/kiel/nm64_kiel_%04i-%02i-%02iT%02i:02:00Z.nm_raw" \
            % (year, month, day, hour)

    try:
        df = pd.read_csv(
            filename,
            delim_whitespace=True,
            header=None,
            names=names,
            )
    except FileNotFoundError:
        print("file not found: %s" %(filename))
        return None
    # TODO: test that 60 rows are read (when hour is complete)
    df.index = pd.to_datetime(df['timestamp'])
    df.index = df.index.tz_localize(None)  # remove TZ

    # do NOT modify BM35 column, this is taken care of in median_editor

    # drop unused columns
    df.drop(["timestamp"], axis=1, inplace=True)

    return(df)


# --------------------------------------------------------------------------
def main():
    """test kiel3 functions"""
    import argparse

    parser = argparse.ArgumentParser(description='NMDB kiel3 data.')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-v', '--verbose', action='count', default=0, help='increase verbosity level')
    group.add_argument("-q", "--quiet", action="store_true")
    parser.add_argument('station', metavar='STATION', nargs='?', default='test', help='NMDB station short name')
    parser.add_argument('data_type', choices=['1m', '1h', 'env', 'meta'], nargs='?', default='1m')
    parser.add_argument('-Y', '--year', type=int, dest='year', help='year')
    parser.add_argument('-M', '--month', type=int, dest='month', help='month')
    parser.add_argument('-D', '--day', type=int, dest='day', help='day')
    parser.add_argument('-n', '--dry-run', dest='dryrun', action="store_true", help='print but do not execute commands')

    #test = 'kiel3 1m -Y 2024 -M 1 -D 1 -n'
    test = 'kiel3 1m -Y 2024 -M 11 -D 12 -n' # data on falabala
    #test = 'kiel3 1m -Y 2023 -M 9 -D 29 -n'
    #test = 'kiel3 1h -Y 2022 -M 7 -D 29 -n'
    #test = 'kiel3 1m -Y 2023 -M 8 -D 29 -n'

    # countrate goes to zero!
    #File "/home/steigies/src/python/site-packages/nmdb/upload.py", line 226, in upload
    #write += " VALUES " + mysql_row(pandas2nmdb(args, row, my_data_type, p_min, p_max,c_min, c_max))
    #                      ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    #File "/home/steigies/src/python/site-packages/nmdb/mysql/mysql.py", line 92, in mysql_row
    #for val in row:
    #TypeError: 'NoneType' object is not iterable
    #test = 'kiel3 1h -Y 2023 -M 6 -D 12 -n'
    #test = 'kiel3 1m -Y 2023 -M 6 -D 12 -n'

    args = parser.parse_args(args=test.split())

    #mode = "raw"
    mode = "rt"
    #mode = "rev"

    print("This is the kiel3 module.")
    print(vars(args))

    year = args.year
    month = args.month
    day = args.day
    station = args.station
    data_type = args.data_type

    if mode in "raw":
        hour = 0
        data = read_raw(year, month, day, hour)

    if mode in "rt":
        data = read_rt(station, data_type, year, month, day)
        if data is not None:
            data.plot(y=["corr", "uncorr"])

    if mode in "rev":
        #data = read_rev(options, station, data_type, year, month, day)
        data = read_rev(station, data_type, year, month, day)

    return data

# --------------------------------------------------------------------------
if __name__ == '__main__':

    DO_main = False
    DO_lvl0 = False
    DO_lvl1 = False
    DO_lvl2 = False

    YEAR = 2019
    DO_main = True
    #DO_lvl0 = True
    #DO_lvl1 = True
    #DO_lvl2 = True

    if DO_main:
        print('main()')
        #data = main()
        ##for line in data:
        ##    print(line)
        ##print(data)
        station = "kiel3"
        print(min_value(station, pressure[station]))
        print(max_value(station, pressure[station]))


    if DO_lvl0:
        for i in range(1, 12+1):
            lvl0("kiel3", YEAR, i)
        #    #lvl0("kiel3", 2024, i)
    if DO_lvl1:
        #for i in range(1, 6+1):
        #    (df) = lvl1("kiel3", 2024, i)
        #    df.p_bm35.plot()
        #    df.p_int.plot()
        for i in range(1, 12+1):
            lvl1("kiel3", YEAR, i)
        #(df) = lvl1("kiel3", 2024, 1, 8)
        #df.p_bm35.plot()
        #df.p_int.plot()
        #df[['p_bm35', 'p_int']].plot()
        # TODO fail to insert rev data on duplicate keys
        # TODO fail to read for missing raw data
    if DO_lvl2:
        (df) = lvl2("kiel3", 2024, 1, 8)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jul 26 10:38:41 2023

@author: steigies
"""

# http://wiki.python.org/moin/ConfigParserExamples
#import configparser  # python-configparser

# --------------------------------------------------------------------------
def config_section_map(section, config):
    """return configuration as dict"""

    dict1 = {}
    options = config.options(section)
    for option in options:
        try:
            dict1[option] = config.get(section, option)
            if dict1[option] == -1:
                # DebugPrint("skip: %s" % option)
                pass
        except:
            print("exception on %s!" % option)
            dict1[option] = None
    return dict1





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 31 14:22:09 2024

@author: steigies
"""

import sys
import pytest

from nmdb.station import kiel3

# --------------------------------------------------------------------------
#def setup_module(module: Any) -> None:
#    print(f"setting up MODULE {module.__name__}")
    #realtime = {"kiel3": kiel3.read_rt}
    #revised = {"kiel3": kiel3.read_rev}
    #pressure = {"kiel3": kiel3.pressure}
    #countrate = {"kiel3": kiel3.countrate}


# --------------------------------------------------------------------------
#def teardown_module(module: Any) -> None:
#    print(f"tearing down MODULE {module.__name__}")



# --------------------------------------------------------------------------
def test_pressure():
    # Given
    # test double index as in upload
    pressure = {"kiel3": kiel3.pressure}
    #pressure = kiel3.pressure
    # Each test case is a tuple of (input, expected_result)
    test_cases = [
        (["kiel3"], [900., 1100.]),
    ]
    for my_station, expected in test_cases:
        # When
        my_station = "kiel3"
        p_min = pressure[my_station][my_station][0]
        p_max = pressure[my_station][my_station][1]
        #p_min = pressure[my_station][0]
        #p_max = pressure[my_station][1]
        # Then
        expected_p_min = expected[0]
        expected_p_max = expected[1]
        assert p_min == expected_p_min
        assert p_max == expected_p_max


# --------------------------------------------------------------------------
def test_countrate():
    # Given
    countrate = {"kiel3": kiel3.countrate}
    # Each test case is a tuple of (input, expected_result)
    test_cases = [
        (["kiel3"], [60., 6000.]),
    ]
    for my_station, expected in test_cases:
        # When
        my_station = "kiel3"
        c_min = countrate[my_station][my_station][0]
        c_max = countrate[my_station][my_station][1]
        # Then
        expected_c_min = expected[0]
        expected_c_max = expected[1]
        assert c_min == expected_c_min
        assert c_max == expected_c_max



#! /usr/bin/env python3
"""NMDB database table formats

Copyright (c) 2011-2019 Christian T. Steigies <steigies@physik.uni-kiel.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"


__all__ = ["ori_format", "rev_format", "hour_format", "meta_format"]

## --------------------------------------------------------------------------

ori_format = (
    "start_date_time, "
    "length_time_interval_s, "
    "measured_uncorrected, "
    "measured_corr_for_efficiency, "
    "measured_corr_for_pressure, "
    "measured_pressure_mbar"
    )

rev_format = (
    "start_date_time, "
    "length_time_interval_s, "
    "revised_uncorrected, "
    "revised_corr_for_efficiency, "
    "revised_corr_for_pressure, "
    "revised_pressure_mbar, "
    "version"
    )

hour_format = (
    "start_date_time, "
    "uncorrected, "
    "corr_for_efficiency, "
    "corr_for_pressure, "
    "pressure_mbar"
    )

meta_format = (
    "start_date_time, "
    "end_date_time, "
    "quality_flag, "
    "comment, "
    "ID_RECORD"
    )

def env_format():    
    """format of the NMDB env tables"""
    return None

## --------------------------------------------------------------------------

if __name__ == "__main__":
    print("This is the nmdb format helper module.")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""KIEL1 (IZMIRAN) data format

Copyright (c) 2009-2024 Christian T. Steigies <steigies@physik.uni-kiel.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

This module offers functions to read NM data in BIN format as used in KIEL.

KIEL BIN data availability:
1994-08-01 -- 2017-05-16
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"

#from datetime import datetime, date, time
#from calendar import monthrange
# http://wiki.python.org/moin/ConfigParserExamples
import configparser  # python-configparser
import pandas as pd
import os
#import MySQLdb  # python-mysqldb abstracted by sqlalchemy
from sqlalchemy import create_engine
import sqlite3

#from nmdb.tools.configtool import config_section_map
#from tools.query import dt_start_stopp

my_station = ["kiel1"]
my_names = {"kiel1": "KIEL1"}
my_station_longnames = {"kiel1": "Kiel (IZMIRAN)"}
pressure = {"kiel1": [900., 1200.]}
countrate = {"kiel1": [10., 1000.]}



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""usage: python status.py -h

NMDB realtime

Copyright (C) 2008-2023 Christian T. Steigies <steigies@physik.uni-kiel.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"

import pandas as pd

__cmd__ = 'realtime'

# --------------------------------------------------------------------------

def parser(subparsers):
    # create the parser for the "realtime" command
    sub_p = subparsers.add_parser(__cmd__, help='get NMDB realtime data')
    group = sub_p.add_mutually_exclusive_group()
    group.add_argument('-v', '--verbose', action='count', default=0, help='increase verbosity level')
    group.add_argument("-q", "--quiet", action="store_true")
    sub_p.set_defaults(cmd=__cmd__)

    return(sub_p)


# --------------------------------------------------------------------------
def realtime(args):
    """read NMDB realtime data into a pandas dataframe"""
    print(vars(args))
    if args.verbose:
        print(vars(args))
    if not args.quiet:
        print("NMDB: realtime data")

    realtime = "http://rt.nmdb.eu/realtime.txt"  # NMDB realtime data is available here    
    columns = ['timestamp', 'station', 'countrate']  # list with column names
    df = pd.read_table(realtime, comment="#", sep=";", header=0, names=columns)

    df['timestamp'] = pd.to_datetime(df['timestamp'])  # convert str to datetime
    df = df.astype({'station':'string'})
    df['countrate'] = pd.to_numeric(df['countrate'], errors='coerce')  # None -> NaN
    df.dropna(inplace=True)  # drop rows that contain NaN/None values
    if args.verbose:
        print(df)

    # create a pivot table with station names as column name
    data = df.pivot(index='timestamp', columns='station', values='countrate')
    # pivot table with max, mean, min for each station
    d3 = df.pivot_table(index='station', values='countrate', aggfunc=('mean', 'max', 'min'))
    print(d3)
    if args.verbose:
        print(data)

    return(data)


# --------------------------------------------------------------------------
def main():
    from matplotlib import pyplot as plt
    import argparse
    p = argparse.ArgumentParser(description='NMDB realtime interface.')
    # add subparsers!
    subparsers = p.add_subparsers()
    parser_realtime = parser(subparsers)

    args = p.parse_args(args=[__cmd__])
    data = realtime(args)
    print(data)
    # run this file in spyder to see the plots
    #data.plot()  # plot all the data
    #data.ATHN.plot()  # plot a single station
    #data[['ATHN', 'OULU']].plot()  # plot multiple stations

    # plt.plot(data)
    # #plt.figtext(0.15, 0.72, f"NMDB realtime data")
    # #plt.xlabel(f"$\omega$ in 1/s")
    # #plt.ylabel(f"Auslenkung in $\mu$m")
    # plt.title("NMDB realtime data")
    # #plt.legend()
    # plt.show()
    # #plt.savefig(f"Realtime.png", dpi=600)
    # plt.close()

    # calculate the mean countrate for one or several stations
    #print("mean countrate for ATHN: ", data.ATHN.mean())
    #print(data[['ATHN', 'OULU']].mean())
    #print("mean countrate:\n", data.mean())
    #print("max countrate:\n", data.max(), data.min())
    #print("min countrate:\n", data.min())
    #d2  = data.mean()
    #df2  = d2.merge(data.max())
    #d2.a = data.max()
    #d2.i = data.min()
    #print(df2)
    #print("countrate:\n", data.mean() + data.max() + data.min())
    # data = data.join(
    # data.groupby('station')['ATHN'].aggregate(['mean', 'min', 'max']),
    # on='station')
    # data["mean"]= data.groupby('statistics')['s_values'].transform('mean')
    #print(data)


# --------------------------------------------------------------------------
if __name__ == '__main__':
    main()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 31 14:22:09 2024

@author: steigies
"""

import sys
import pytest

from nmdb.station import aata
from nmdb.station import drhm
from nmdb.station import kiel3

#pressure = {"aata": aata.pressure,
##            "drhm": drhm.pressure,
#            "kiel0": kiel3.pressure,
#            "kiel1": kiel3.pressure,
#            "kiel2": kiel3.pressure,
#            "kiel3": kiel3.pressure,}

pressure = kiel3.pressure
#yaata.pressure
#pressure.update(kiel3.pressure)
# --------------------------------------------------------------------------
#def setup_module(module: Any) -> None:
#    print(f"setting up MODULE {module.__name__}")
    #realtime = {"kiel3": kiel3.read_rt}
    #revised = {"kiel3": kiel3.read_rev}
    #pressure = {"kiel3": kiel3.pressure}
    #countrate = {"kiel3": kiel3.countrate}


# --------------------------------------------------------------------------
#def teardown_module(module: Any) -> None:
#    print(f"tearing down MODULE {module.__name__}")



# --------------------------------------------------------------------------
def test_pressure():
    # Given
    # test double index as in upload
    #pressure = {"kiel3": kiel3.pressure}
    #pressure = kiel3.pressure
    # Each test case is a tuple of (input, expected_result)
    test_cases = [
        (["aata", "aata"], [900., 1100.]),
        (["kiel3", "kiel3"], [900., 1100.]),
    ]
    for my_station, expected in test_cases:
        # When
        institute = my_station[0]
        station = my_station[1]
        p_min = pressure[institute][station][0]
        p_max = pressure[institute][station][1]
        # Then
        expected_p_min = expected[0]
        expected_p_max = expected[1]
        assert p_min == expected_p_min
        assert p_max == expected_p_max

# --------------------------------------------------------------------------
if __name__ == '__main__':
    print(pressure)
    for station in ["kiel1", "kiel2", "kiel3"]:
        print(pressure[station])


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""read leik (LEIK) data into a pandas dataframe -- testing station

Copyright (c) 2023 Christian T. Steigies <steigies@physik.uni-kiel.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"

import pandas as pd

from datetime import datetime
from time import gmtime, strftime

# TODO use your station names here
stations = ["leik"]
#names = {"leik": "LEIK"}
station_longnames = {"leik": "Test Station"}




# --------------------------------------------------------------------------
def read_rt(options, station, data_type, year, month, day):
    """read realtime data for one day at a time

    options     verbose, dryrun, ...
    station     station short name, must be valid station
    data_type   "ori", "1h", "env", "meta"
    year        integer (1950-9999), may not be None
    month       integer (1-12), if None, read month 1-12
    day         integer (1-31), if None, read day 1-31

    return list of values in nmdb format, all countrates are counts/s
    [str2datetime(str(start_date_time)), # start of the measurement
     60,      # length of the measurement in seconds: 60 for 1min data
     uncorr,  # countrate of the whole monitor, uncorrected
     ceff,    # countrate corrected for pressure and efficiency (often == corr)
     corr,    # countrate corrected for pressure
     press]   # atmospheric pressure in mbar
    """
    data = []
    for hour in range(0, 24):
        data += read_1m(options, station, year, month, day, hour)

    return data


# --------------------------------------------------------------------------
def read_rev(options, station, data_type, year, month=None, day=None):
    """read revised data

    options     verbose, dryrun, ...
    station     station short name, must be valid station
    data_type   "ori", "1h", "env", "meta"
    year        integer (1950-9999), may not be None
    month       integer (1-12), if None, read month 1-12
    day         integer (1-31), if None, read day 1-31

    return list of values in nmdb format, all countrates are counts/s
    [str2datetime(str(start_date_time)),
     60,      # length of the measurement in seconds: 60 for 1min data
     uncorr,  # countrate of the whole monitor, uncorrected
     ceff,    # countrate corrected for pressure and efficiency (often == corr)
     corr,    # countrate corrected for pressure
     press]   # atmospheric pressure in mbar
    """

    if station in stations:
        pass
    else:
        raise ValueError  # wrong station

    data = []
    if year is None:
        raise ValueError  # year has to be specified
    elif month is None:  # read one year
        for m in range(1, 13):
            data += read_rev(options, station, data_type, year, month=m)
    elif day is None:  # read one month
        for d in range(1, 32):
            if valid_date(year, month, d):
                data += read_rev(options, station, data_type, year, month, day=d)
    else:  # read one day
        for hour in range(0, 24):
            if data_type in "1h":
                data += read_1h(options, station, year, month, day, hour)
            elif data_type in "ori":
                data += read_1m(options, station, year, month, day, hour)
            elif data_type in ["env", "meta"]:
                print("data_type '{}' not implemented yet".format(data_type))
                raise ValueError 
            else:
                print("unknown data_type: '{dt}'".format(dt=data_type))
                raise ValueError 
    return data


# --------------------------------------------------------------------------
def read_1h(options, station, year, month, day, hour):
    """"read one hour of data with 1-hour resolution
    just returning some dummy data
    """
    data = []
    start_date_time = datetime(year, month, day, hour)
    data.append([start_date_time, 3600, 99.1, 102.0, 100.0, 1013.1])

    return data


# --------------------------------------------------------------------------
def read_1m(options, station, year, month, day, hour):
    """"read one hour of data with 1-minute resolution
    just returning some dummy data
    """
    data = []
    for minute in range(0, 60):
        start_date_time = datetime(year, month, day, hour, minute)
        data.append([start_date_time, 60, 99.1, 102.0, 100.0, 1013.1])

    return data


# --------------------------------------------------------------------------
if __name__ == '__main__':
    import optparse
    parser = optparse.OptionParser("usage: %prog [options]")
    parser.add_option("-v", dest="verbose", default=0, action="count",
            help="increment output verbosity; may be specified multiple times")
    (options, args) = parser.parse_args()

    print("This is the sample module.")

    station = "smpl"
    data_type="ori"
    #data_type="1h"
    #data_type="meta"
    year = 2023
    month = 1
    day = 1
    hour = 12

    df = read_pandas(options, station, data_type, year, month, day, online=False)
    print(df)
    #exit(0)
    
    ##data = read_1m(options, station, year, month, day, hour)
    ##data = read_1h(options, station, year, month, day)
    #data = read_rev(options, station, data_type, year, month, day)
    data = read_rt(options, station, data_type, year, month, day)

    for line in data:
        print(line)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""KIEL2 data format

Copyright (c) 2011-2019 Christian T. Steigies <steigies@physik.uni-kiel.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

This module offers functions to read NM data from the Alcala database as used in KIEL
and return data in the format as used by NMDB.

KIEL2 (Alcala) data availability:
2011-01-12 -- 2017-07-18
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"

# http://wiki.python.org/moin/ConfigParserExamples
import configparser  # python-configparser
import numpy as np
import pandas as pd
import os
#import MySQLdb  # python-mysqldb abstracted by sqlalchemy
from sqlalchemy import create_engine
import sqlite3

#import sys
#print(sys.path)

from nmdb.tools.query import dt_start_stopp
from nmdb.tools.configtool import config_section_map

my_station = ["kiel2"]
my_names = {"kiel2": "KIEL2"}
my_station_longnames = {"kiel2": "Kiel (alcala)"}
pressure = {"kiel2": [900., 1100.]} # (961.8 - 1041.05) mbar
countrate = {"kiel2": [60., 6000.]} # typical counts per tube (450-600) / min

TUBES = 18 # Kiel is an 18NM64

# one hour long-term averages
# taken from original IZMIRAN software
n0 = [#0, # first tube is ch_01 not 0
      30220, 35780, 36960, 37362, 37020, 31690,
      32550, 36785, 38000, 37284, 36826, 32270,
      32200, 37003, 37685, 37910, 37125, 31030]

# -------------------------------------------------------------------
def read_nmdadb(year, month=None, day=None):
    """
    read data from Alcala nmdadb and store it in sqlite files
    nmdadb table can be deleted, local mysql server not needed anymore
    """

    return None  # sqlitefiles have been created, do NOT overwrite them

    my_station = "kiel2_nmdadb"
    my_rcfile = "~/.nmdbrc"

    config = configparser.ConfigParser()
    try:
        rcfile = os.path.expanduser(my_rcfile)
        config.read(rcfile)
    except:
        print("parse_config: rcfile not found:", my_rcfile)
        raise ValueError

    username = config_section_map(my_station, config)['username']
    password = config_section_map(my_station, config)['password']
    hostname = config_section_map(my_station, config)['hostname']
    dbname = config_section_map(my_station, config)['dbname']
    #portnumber = config_section_map(my_station, config)['port']

    db_connection_str = 'mysql+pymysql://%s:%s@%s/%s' \
        % (username, password, hostname, dbname)
    db_connection = create_engine(db_connection_str)

    table = "binTable"
    (start, stopp) = dt_start_stopp(year, month, day)
    print(year, month, day, start, stopp)
    query = "SELECT * FROM %s WHERE start_date_time BETWEEN '%s' AND '%s'" \
        %(table, start, stopp)
    print(query)

    try:
        df = pd.read_sql(query, con=db_connection)
    except Exception as error:
        print(error)
        return None

    filename = "/data/nm64/kiel2/kiel2_%i.raw.sqlite" % (year)
    try:
        con = sqlite3.connect(filename)
    except:
        return None
    df.to_sql(name='data', if_exists='replace', con=con)

    return df

# --------------------------------------------------------------------------
# TODO nearly identical to kiel3 function
def read_lvl(station, lvl, year, month, day=None):
    """
    read data for lvl from sqlite
    """
    # TODO DRY! put this in a function, lvl as parameter
    if station in my_station:
        pass
    else:
        raise ValueError  # wrong station

    # sqlite file
    if lvl < 0:  # raw data, monthly files
        filename = "/data/nm64/kiel2/%s_%04i.raw.sqlite" \
            % (station, year)
        table = 'data'
    else:  # all levels of revised data, yearly files
        filename = "/data/nm64/kiel2/%s_%04i.rev.sqlite" \
            % (station, year)
        table = 'lvl%i' %(lvl)
    try:
        con = sqlite3.connect(filename)
    except:
        return None

    (start, stopp) = dt_start_stopp(year, month, day)
    if lvl < 0:
        query = "SELECT * FROM '%s' WHERE start_date_time BETWEEN '%s' AND '%s'" %(table, start, stopp)
    else:
        query = "SELECT * FROM '%s' WHERE datetime BETWEEN '%s' AND '%s'" %(table, start, stopp)
    print(query)
    try:
        df = pd.read_sql_query(query, con)
    except Exception as error:
        print(error)
        return(None)

    if lvl < 0:
        df['datetime'] = df['start_date_time']
        df.drop(['start_date_time', 'index'], axis=1, inplace=True) # drop obsolete columns
    df.index = pd.to_datetime(df['datetime'])
    df.index = df.index.tz_localize(None)  # remove TZ
    df.drop(['datetime'], axis=1, inplace=True) # drop obsolete columns

    return(df, con)

# -------------------------------------------------------------------
def lvl0(station, year, month=None, day=None):
    """
    create sqlite database with lvl0 data
    """
    if station in my_station:
        pass
    else:
        raise ValueError  # wrong station

    (df, con) = read_lvl(station, -1, year, month, day)
    con.close()

    df['press'] = df['atmPressure']/100.
    df.drop(['atmPressure', 'hv1', 'hv2', 'hv3', 'temp_1', 'temp_2'], axis=1, inplace=True) # drop obsolete columns

    # write to sqlite
    filename = "/data/nm64/kiel2/%s_%04i.rev.sqlite" % (station, year)
    try:
        con = sqlite3.connect(filename)
    except:
        return None
    #df.to_sql(name='lvl0', if_exists='append', con=con)
    df.to_sql(name='lvl0', if_exists='replace', con=con)
    con.close()

    return(df)

# --------------------------------------------------------------------------
def interpolate_pressure(df, ts):
    minutes = 2  # +- 2 minutes
    t_min = ts - pd.Timedelta(minutes=minutes)
    t_max = ts + pd.Timedelta(minutes=minutes)
    df2 = df[t_min:t_max][['press']]
    pressure = df2['press'].median()
    #print(pressure)

    return(pressure)

# --------------------------------------------------------------------------
def lvl1(station, year, month=None, day=None):
    """
    create lvl1 data
    find tubes with countrate out of range and set to NA
    find pressure out of range and set to NA
    """

    p_min = pressure["kiel2"][0]
    p_max = pressure["kiel2"][1]
    cr_min = countrate["kiel2"][0]
    cr_max = countrate["kiel2"][1]

    (df, con) = read_lvl(station, 0, year, month, day)

    # Verify that result of SQL query is stored in the dataframe
    print(df.head())

    # BM35 barometer
    df.loc[df['press'] < p_min,'press'] = np.nan  # pressure too low
    df.loc[df['press'] > p_max,'press'] = np.nan  # pressure too high

    # this should be done in lvl2:
    # dataframe containing BM35 pressure measurements with errors:
    df_p = df.loc[pd.isna(df['press'])]
    print(df_p)
    for idx, row in df_p.iterrows():
        press = interpolate_pressure(df, idx)
        df.at[idx, 'press'] = press  # replace missing value
        print("replacing press at", idx, "from", row.press, "with", press)

    # eliminate counts that are clearly out of range for this station
    for i in range(1, TUBES+1):
        ch = "ch%02i" % (i)
        df.loc[df[ch] < cr_min,ch] = np.nan  # countrate too low
        df.loc[df[ch] > cr_max,ch] = np.nan  # countrate too high

    # TODO change dtypes of ch to INT instead of float64
    #df.to_sql(name='lvl1', if_exists='append', con=con)
    df.to_sql(name='lvl1', if_exists='replace', con=con)
    con.close()

    return(df)

# --------------------------------------------------------------------------
if __name__ == '__main__':
    DO_read = False
    DO_lvl0 = False
    DO_lvl1 = False
    DO_lvl2 = False

    YEAR = 2011
    #DO_read = True
    #DO_lvl0 = True
    DO_lvl1 = True
    #DO_lvl2 = True

    station = "kiel2"
    data_type = "1m"

    if DO_read:
        df = read_nmdadb(year=2011, month=1, day=15)
        print(df)

    if DO_lvl0:
        for year in range(2011, 2017+1):
            df = lvl0(station, year)
            print(df)

    if DO_lvl1:
        #df = lvl1(station, 2011,1,15)
        #print(df)
        for year in range(2011, 2017+1):
            df = lvl1(station, year)
            print(df)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 13 16:55:54 2023

@author: steigies

https://stackoverflow.com/questions/14745022/how-to-split-a-dataframe-string-column-into-two-columns
"""

import pandas as pd

def main():
    df = pd.DataFrame({'AB': [':BM35:989.46:BM35b:989.28:', ':BM35:989.43:BM35b:989.25:',
                          ':BM35:1004.7:', ':BM35:1028.32:GB1:0.0:']})

    for index, row in df.iterrows():
        #print(row[0])
        #p_bm35 = pressure(row[0])
        p_bm35 = press_parse(row[0], "BM35")
        #p_bm = p_bm35(row['p_bm35'])
        print(p_bm35)
        print()

#df['AB_split'] = df['AB'].str.split('BM35')
#print(df)

def pressure(bm35):
    """find :BM35: in string. Numbers until next : are p_bm35"""
    p_bm35 = None
    head = bm35.find(":BM35:")
    if head >= 0:
        tail = bm35[head+6:].find(":")
        p_bm35 = float(bm35[head+6:tail+6])

    return p_bm35

    
def press_parse(press, key):
    print(press, key)

    value = 0.  # no pressure, store as 0
    if isinstance(press, float):  # until 2022 a single BM35 value was used
        value = float(press)
    else:
        pressure = {}
        if press[0] == ":":  # several key:value pairs are stored
            data = press.split(":")
            keyword = data.pop(0)  # pop off first ":"
            while len(data) > 1:  # need two values: keyword and value
                keyword = data.pop(0)
                value = data.pop(0)
                pressure[keyword] = value

        #print(pressure, key, pressure[key])
        try:
            value = pressure[key]  # return pressure for the given key
        except KeyError:  # return 0 if key not found
            value = 0.

    return value


# --------------------------------------------------------------------------
if __name__ == '__main__':
    main()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 31 14:22:09 2024

@author: steigies
"""

import sys
import pytest

def test_simple_skip() -> None:
    if sys.platform != "ios":
        pytest.skip("Test works only on ios")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 31 14:22:09 2024

@author: steigies
"""

from __future__ import annotations
from typing import Any, Callable

#import pytest
import pandas as pd
#import sys

#from nmdb.tools.query import dt_start_stopp
from query import dt_start_stopp

# --------------------------------------------------------------------------
def setup_module(module: Any) -> None:
    print(f"setting up MODULE {module.__name__}")


# --------------------------------------------------------------------------
def teardown_module(module: Any) -> None:
    print(f"tearing down MODULE {module.__name__}")


# --------------------------------------------------------------------------
#def test_simple_skip() -> None:
#    if sys.platform != "ios":
#        pytest.skip("Test works only on ios")


# --------------------------------------------------------------------------
def test_dt_start_stopp():
    # Given
    #station = "kiel2"
    # Each test case is a tuple of (input, expected_result)
    test_cases = [
        ([2011, 1, 1], ["2011-01-01 00:00:00", "2011-01-01 23:59:59"]),
        ([2011, 1, None], ["2011-01-01 00:00:00", "2011-01-31 23:59:59"]),
        ([2011, None, None], ["2011-01-01 00:00:00", "2011-12-31 23:59:59"]),
    ]
    for date, expected in test_cases:
        # When
        year = date[0]
        month = date[1]
        day = date[2]
        (start, stopp) = dt_start_stopp(year, month, day)
        # Then
        expected_start = expected[0]
        expected_stopp = expected[1]
        assert start == expected_start
        assert stopp == expected_stopp






# -*- coding: utf-8 -*-

# --------------------------------------------------------------------------
def dt_start_stopp(year, month, day):
    if month is None:
        month_min = 1
        month_max = 12
        day_min = 1
        day_max = 31
    else:
        month_min = month
        month_max = month
        if day is None:
            day_min = 1
            day_max = 31
        else:
            day_min = day
            day_max = day
    # read into dataframe
    start = "%04i-%02i-%02i 00:00:00" % (year, month_min, day_min)
    stopp = "%04i-%02i-%02i 23:59:59" % (year, month_max, day_max)

    return(start, stopp)





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
find table name for station:
    kiel3 is stored in kiel2 table (merge all to kiel later...)
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"


def station_name(station):
    if station == "kiel3":
        value = "kiel2"
    else:
# TODO check is staion is a valid station?
        value = station

    return value




#! /usr/bin/env python3

from nmdb.parser import nmdb_parser

# --------------------------------------------------------------------------
def main():
    """Entry point for the application script"""
    parser = nmdb_parser()
    # add subparsers!
    args = parser.parse_args()
    #args.func(args)

    if args.verbose > 2:
        print(f"Running '{__file__}'")
    if args.verbose:
        print(args)

    if args.cmd is None:
        parser.print_help()

    if (args.cmd == "status"):
        print("calling status")
    elif (args.cmd == "realtime"):
        print("calling realtime")
        from nmdb.realtime import realtime
        data = realtime(args)
        print(data)
    elif (args.cmd == "download"):
        print("calling download")
    elif (args.cmd == "upload"):
        from nmdb.upload import upload
        upload(args)
    elif (args.cmd == "revise"):
        print("calling revise")
    elif (args.cmd == "alert"):
        print("calling alert")
    else:
        print("command not recognized")
    
    if not args.quiet:
        print("Ready.")


# --------------------------------------------------------------------------
if __name__ == '__main__':
    main()

# BUG unset PYTHONPATH otherwise old nmdb versions will be executed...
# but without PYTHONPATH nmrena_kiel3.sh does not work (used nmdb3)...
# TODO convert nmrena import script, include in nmdb tool



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""usage: python upload3.py -v -f nmdbrc newk ori|1h

NMDB upload3: upload real-time data for all stations, python3/pandas version

Copyright (C) 2008-2024 Christian T. Steigies <steigies@physik.uni-kiel.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"

# --------------------------------------------------------------------------
# on centos: yum install
#   MySQL-python
#   numpy???
#   python-sqlite

#import sys
# http://wiki.python.org/moin/ConfigParserExamples
import configparser  # python-configparser
from datetime import datetime
from datetime import timedelta
import MySQLdb  # python-mysqldb
import os
import pandas as pd
import numpy as np
##import sqlite3

from nmdb.tools.configtool import config_section_map
from nmdb.tools.datetool import last_datapoint
from nmdb.tools.datetool import nmdb_timestamp
from nmdb.tools.datetool import sys_timestamp
#from nmdb.tools.datetool import str2datetime

##from nmdb3.nmdb.sqlite import SqlData
from nmdb.mysql.mysql import query #, nmdb_timestamp
from nmdb.mysql.mysql import mysql_row
from nmdb.mysql.mysql import write_nmdb

from nmdb.station import aata
from nmdb.station import drhm
from nmdb.station import kiel3

realtime = {"aata": aata.read_rt,
            "drhm": drhm.read_rt,
            "mtws": drhm.read_rt,
            "ldvl": drhm.read_rt,
            "hlea": drhm.read_rt,
            "clmx": drhm.read_rt,
            "huan": drhm.read_rt,
            "kiel3": kiel3.read_rt,}

#revised = {"aata": aata.read_rev,
#           "drhm": drhm.read_rev,
#           "kiel": kiel.read_rev,}

#pressure = {"aata": aata.pressure,
#            "drhm": drhm.pressure,
#            "kiel3": kiel3.pressure,}

#countrate = {"aata": aata.countrate,
#             "drhm": drhm.countrate,
#             "kiel3": kiel3.countrate,}


# --------------------------------------------------------------------------
def parser_upload(subparsers):
    # create the parser for the "upload" command
    sub_p = subparsers.add_parser('upload', help='upload to NMDB')
    sub_p.add_argument('station', metavar='STATION', nargs='?', default='test', help='NMDB station short name')
    sub_p.add_argument('data_type', choices=['1m', '1h', 'env', 'meta'], nargs='?', default='1m')
    sub_p.add_argument('-n', '--dry-run', dest='dryrun', action="store_true", help='print but do not execute commands')
    sub_p.add_argument('--file', dest='rcfile', default='~/.nmdbrc', help='config file for NMDB')
    sub_p.add_argument('-H', '--host', dest='hostname', default='write.nmdb.eu', help='NMDB hostname to connect to')
    sub_p.add_argument('-P', '--port', type=int, dest='port', default='3306', help='NMDB port number to connect to')
    sub_p.add_argument('-Y', '--year', type=int, dest='year', help='year')
    sub_p.add_argument('-M', '--month', type=int, dest='month', help='month')
    sub_p.add_argument('-D', '--day', type=int, dest='day', help='day')
    sub_p.add_argument('-T', '--timeout', type=int, dest='timeout', default=30, help='timeout')
    sub_p.add_argument('--maxcount', type=int, dest='maxcount', default=120, help='send max values')
    sub_p.add_argument('--start', dest='tstart', default='2020-01-01 00:00:00', help='use this start date if there is no data for this station in NMDB')
    group = sub_p.add_mutually_exclusive_group()
    group.add_argument('-v', '--verbose', action='count', default=0, help='increase verbosity level')
    group.add_argument("-q", "--quiet", action="store_true")
    sub_p.set_defaults(cmd="upload")

    return(sub_p)


# --------------------------------------------------------------------------
def upload(args):
    print("This is NMDB upload.")

    if args.verbose > 1:
        #print("args:", args)
        print(vars(args))
    if args.dryrun:
        print("dryrun: not sending any data to NMDB")
    my_rcfile = args.rcfile
    my_station = args.station
    my_data_type = args.data_type

    if my_data_type in ["1m", "1h"]:
        pass
    elif my_data_type in ["env", "meta"]:
        raise ValueError  # not yet implemented
    else:
        raise ValueError  # illegal data_type

    config = configparser.ConfigParser()
    try:
        rcfile = os.path.expanduser(my_rcfile)
        config.read(rcfile)
    except:
        print("parse_config: rcfile not found:", my_rcfile)
        raise ValueError

    username = config_section_map(args.station, config)['username']
    password = config_section_map(args.station, config)['password']

    #print("upload for %s" % (my_station))
    #print("Sending real-time %s data for %s (%s)" % (data_type, username, longname))
    #print("Sending real-time %s data for %s using %s"
    #      % (my_data_type, my_station, my_rcfile))

    try:
        read_rt = realtime[my_station]
        # TODO def pressure and countrate function with station as argument
        #p_min = pressure[my_station][my_station][0]
        #p_max = pressure[my_station][my_station][1]
        #c_min = countrate[my_station][my_station][0]
        #c_max = countrate[my_station][my_station][1]
        p_min = 100.
        p_max = 1500.
        c_min = 1.
        c_max = 1000000.

    except KeyError:
        print("No import filter for %s available" % (my_station))
        raise ValueError  # station undefined
    except:
        raise ValueError  # unknown error

    # upload is executed every minute, so we need to timeout after ~30 seconds
    start = pd.to_datetime(datetime.now())
    timeout = start + timedelta(seconds=args.timeout)

    # open connection to NMDB
    con = MySQLdb.connect(host=args.hostname, port=args.port, 
                          db="nmdb", user=username, passwd=password)
    nmdb = con.cursor()

    # sys_time should be identical to nmdb_time
    # TODO warning/error if not
    sys_time = pd.to_datetime(sys_timestamp())
    nmdb_time = pd.to_datetime(nmdb_timestamp(nmdb))
    # find timestamp of last real-time value
    last_data = last_datapoint(nmdb, my_data_type, my_station)
    # TODO for debugging only go back one year
    #last_data = last_data - timedelta(days=365)
    if args.verbose > 0:
        print("system time\t", sys_time, "\t",)
        print("NMDB time\t", nmdb_time, "\t",)
        print("diff:\t", sys_time - nmdb_time)
        print("last data in NMDB", last_data)
    # only write data newer than last available data
    # for (revising) older data, use revise and station2sql
    #tstart = str2datetime(args.tstart)
    tstart = pd.to_datetime(args.tstart)
    if tstart > last_data:
        last_data = tstart
    print("start writing at:", last_data)
    # TODO if last data is one minute old, wait a bit before trying to read new RT data

    # read data starting with day of last_data
    next_data = last_data.date()
    count = 0  # counting the number of values written
    version = 0  # real-time data is always version 0
    #done = False
    data = []
    # read new day until maxvalues, timeout or two days or no data?
    while True:  # read in two days of data
        if len(data) >= 2:
            break  # two days of data read in, we are done reading in data
        if next_data > nmdb_time.date():
            break  # do not try to read data from the future, we are done
        #print("reading for ", next_data, "days read:", len(data)) 
        try:
            df = read_rt(my_station, my_data_type, next_data.year, next_data.month, next_data.day)
            if df is None:
               if args.verbose:
                    print("no data for ", next_data)
            else:
                #print(df)
                data.append(df)
            continue  # no data, try next day

            #df = df[df.index > last_data-timedelta(hours=1)]  # only data not yet in NMDB
            #df = df.replace({np.nan: None})  # store None in MySQL, not NaN
            #print(my_station, my_data_type, next_data.year, next_data.month, next_data.day)
        except:  # file not found
            if args.verbose:
                print("no data for ", next_data)
            continue  # file may be missing, try next day
        finally:
            next_data = next_data + timedelta(days=1)

    #print("done reading", next_data, "days read:", len(data))
    #print("last_data", last_data, type(last_data))
    #print("nmdb_time", nmdb_time, type(nmdb_time))

    if len(data) > 0:  # no-op when no data
        #print("df_index ", df.index[0], type(df.index[0]))
        df = pd.concat(data)  # dataframe with Kiel3 data in NMDB format
        df = df[df.index > last_data]  # only data not yet in NMDB
        df = df[df.index < nmdb_time]  # no data from the future
        df = df.replace({np.nan: None})  # store None in MySQL, not NaN

        ## TODO BUG in dryrun mode nmdb_date is not set for MXCO only?
        ## write data to NMDB
        for row in df.itertuples():
            # TODO check for valid datetime
            #print(my_station, row)
            write = query(my_station, version, my_data_type)
            write += " VALUES " + mysql_row(pandas2nmdb(args, row, my_data_type, p_min, p_max,c_min, c_max))
            count += write_nmdb(args, nmdb, write)
            if count >= args.maxcount:
                if not args.quiet:
                    print("sent %i values, pausing..." % (args.maxcount))
                break
            if datetime.now() > timeout:
                if not args.quiet:
                    print("timeout of %i seconds reached." % (args.timeout))
                    break
    con.commit()
    print("last data in NMDB", last_datapoint(nmdb, my_data_type, my_station))
    con.close()

    return


# --------------------------------------------------------------------------
def pandas2nmdb(args, row, data_type, p_min, p_max,c_min, c_max):
    """convert one row from dataframe to NMDB format
    """

    if data_type in ["ori"]:
        raise ValueError  # must be called as "1m"
    elif data_type in ["1m", "1h"]:
        #dt = row.Index  # Mariadb does not like TZ in datetime
        # https://stackoverflow.com/questions/10944047/how-can-i-remove-a-pytz-timezone-from-a-datetime-object
        dt = row.Index.replace(tzinfo=None)
        interval = row.interval
        ucorr = row.uncorr
        corr_e = row.ceff
        corr_p = row.corr
        press = row.press

        # test if at least one measured value is available
        if (ucorr or corr_e or corr_p or press) is None:
            if args.verbose:
                print("not writing empty data", row)
            return None

        if (press < p_min) or (press > p_max):
            if args.verbose:
                print("not writing unrealistic pressure", press)
            return None
        if (ucorr < c_min) or (ucorr > c_max):
            if args.verbose:
                print("not writing unrealistic ucorr", ucorr)
            return None
        if (corr_e < c_min) or (corr_e > c_max):
            if args.verbose:
                print("not writing unrealistic corr_e", corr_e)
            return None
        if (corr_p < c_min) or (corr_p > c_max):
            if args.verbose:
                print("not writing unrealistic corr_p", corr_p)
            return None

        if data_type in ["1m"]:
            if dt.second == 0:
                return [str(dt), interval, ucorr, corr_e, corr_p, press]
            else:
                if args.verbose:
                    print("skipping invalid time for 1m data:", dt)
            return None
        elif data_type in ["1h"]:  # no interval for 1h data
            if (dt.minute == 0) and (dt.second == 0):
                return [str(dt), ucorr, corr_e, corr_p, press]
            else:
                if args.verbose:
                    print("skipping invalid time for 1h data:", dt)
            return None
        else:
            raise ValueError  # invalid mode
    elif data_type in ["env", "meta"]:
        raise ValueError  # not yet implemented
    else:
        raise ValueError  # invalid mode while writing


# --------------------------------------------------------------------------
if __name__ == '__main__':
    from nmdb.parser import nmdb_parser
    parser = nmdb_parser()

    #test = 'upload kiel3 1m --dry -v -Y 2023 -M 1'
    #test = 'upload kiel3 1m --dry -v -Y 2024 -M 11'
    #test = 'upload kiel3 1h --dry -v'
    test = 'upload drhm 1m --dry -v'
    #test = 'upload hlea 1m --dry -v -Y 2024 -M 11'

    args = parser.parse_args(args=test.split())
    print(vars(args))
    upload(args)



#!/usr/bin/pytest-3 -v
# -*- coding: utf-8 -*-
"""
Created on Fri Jul 23 14:20:57 2021

@author: steigies

upgrade to python3-tz_2023.3.post1-2_all.deb to get rid of annoying warning
"""

from __future__ import annotations
from typing import Any, Callable

import pytest
#import numpy as np
import pandas as pd
from kiel3 import p_parser, median_editor, tube2ch


# --------------------------------------------------------------------------
def setup_module(module: Any) -> None:
    print(f"setting up MODULE {module.__name__}")


# --------------------------------------------------------------------------
def teardown_module(module: Any) -> None:
    print(f"tearing down MODULE {module.__name__}")


# --------------------------------------------------------------------------
def test_tube2ch() -> None:
    # Given
    station = "kiel"
    # Each test case is a tuple of (input, expected_result)
    test_cases = [
        (1, 2),
        (2, 3),
        (3, 4),
        (4, 5),
        (5, 6),
        (6, 7),
        (7, 10),
        (8, 11),
        (9, 12),
        (10, 13),
        (11, 14),
        (12, 15),
        (13, 18),
        (14, 19),
        (15, 20),
        (16, 21),
        (17, 22),
        (18, 23),
    ]
    for tube, expected in test_cases:
        # When
        output = tube2ch(station, tube)
        # Then
        assert output == expected


# --------------------------------------------------------------------------
def test_tube2_out_of_bounds():
    # Given
    station = "kiel"
    # Each test case is a tuple of (input, expected_result)
    test_cases = [
        (-1, ValueError),
        (0, ValueError),
        (19, ValueError),
        (24, ValueError),
    ]
    for tube, expected in test_cases:
        with pytest.raises(expected):
            print(tube, expected)
            tube2ch(station, tube)


# --------------------------------------------------------------------------
def test_tube2_not_kiel():
    # Given
    station = "oulu"
    # Each test case is a tuple of (input, expected_result)
    test_cases = [
        (-1, ValueError),
        (0, ValueError),
        (1, ValueError),
        (2, ValueError),
        (17, ValueError),
        (18, ValueError),
        (19, ValueError),
        (24, ValueError),
    ]
    for tube, expected in test_cases:
        with pytest.raises(expected):
            tube2ch(station, tube)


# --------------------------------------------------------------------------
def test_p_parser() -> None:
    # Given
    # Each test case is a tuple of (input, expected_result)
    test_cases = [
        (1013.1, 1013.1),                        # very old format: float
        (":BM35:991.55:", 991.55),               # old format
        (":BM35:991.58:BM35b:991.40:", 991.58),  # format since 2023-10
        (":BM35:1017.61:GB1:0.0:", 1017.61),     # format since 2024-01-08
        (":BM35:977.37:BM35b:977.19:", 977.37),  #
    ]
    for pressure, expected in test_cases:
        # When
        output = p_parser(pressure)
        #print("test:", pressure, expected, output)
        # Then
        assert output == expected


# --------------------------------------------------------------------------
def test_median():
    # Given
    ts = pd.to_datetime("2023-06-12 23:56:05")

    channel = []
    for i in range(0, 24):
        channel.append("ch%02i" % (i))
    channel.append("p_bm35")

    data1 = [0, 10, 20, 30, 40, 50, 60, 70,
            600, 601, 602, 603, 604, 605, 606, 607,
            2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 1013.1]

    # RuntimeWarning: All-NaN slice encountered
    data2 = [0, 1, 2, 3, 4, 5, 6, 7,
             0, 1, 2, 3, 4, 5, 6, 7,
             2, 3, 4, 5, 6, 7, 8, 9, 1013.1]

    row = pd.Series(data=data2, index=channel)
    print("row:", row)
    print(median_editor(ts, row))


# --------------------------------------------------------------------------
def notest_median() -> None:
    ts = pd.to_datetime("2023-06-12 23:56:05")

    channel = []
    for i in range(1, 18+1):
        channel.append("ch%02i" % (i))
    channel.append("p_bm35")
    # append integration?
    print(channel)

    # Each test case is a tuple of (input, expected_result)
    test_cases = [([600, 600, 600, 600, 600, 600,
                    600, 600, 600, 600, 600, 600,
                    600, 600, 600, 600, 600, 600,
                    ":BM35:1017.61:GB1:0.0:"], 600.),
                  ]

    for data_in, expected in test_cases:
        data = pd.Series(data=data_in, index=channel)
        print(data)
        # When
        output = median_editor(ts, data_in)
        # Then
        assert output == expected





import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
import os

_ROOT = os.path.abspath(os.path.dirname(__file__))

def main():
    """
    plot GLE70 percentage increase to use as coverpage for proceedings
    """

    sns.set_context("paper")
    sns.color_palette("vlag", as_cmap=True)
    download = os.path.join(_ROOT, 'gle70.dat')
    #download = "gle70.dat"
    # data as downloaded from NEST by selecting:
    # - all stations, GLE70, ASCII data
    # manually edited:
    # - STATION NAMES replaced by 'date' plus station names , separated by ";"
    df = pd.read_table(download, sep=";", comment="#")
    df.index = pd.to_datetime(df['date'])
    # drop unused columns
    df.drop(["date"], axis=1, inplace=True)

    # replace '   null' string for missing data with None
    df = df.replace(to_replace=r'null', value=None, regex=True)
    # values in columns that had 'null's need to be converted to float
    df = df.astype(float)

    # average over 5 minutes
    data = df.rolling(5).mean()

    # station names
    stations = data.columns.values.tolist()

    # make baseline
    sbas = data.head(60) # take the first 60 minutes

    # create percentage with offset
    off = 0
    for s in stations:
        data[s] = ((data[s] - sbas[s].mean())/sbas[s].mean())*100 + off
        off += 10

    lw=5
    ls="solid"
    for s in stations:
        ax = sns.lineplot(data=data[s], linestyle=ls, linewidth=lw, legend=False)

    # hide axis labels and numbers
    ax.set(xlabel=None)
    ax.set(xticklabels=[])
    ax.set(ylabel=None)
    ax.set(yticklabels=[])

    # save images in current directory without overwriting existing files:
    i = 0
    filename = "conf2022_coverpage"
    while os.path.exists('{}{:d}.eps'.format(filename, i)):
        i += 1
    plt.savefig('{}{:d}.eps'.format(filename, i))

    i = 0
    filename = "conf2022_coverpage"
    while os.path.exists('{}{:d}.png'.format(filename, i)):
        i += 1
    plt.savefig('{}{:d}.png'.format(filename, i), dpi=300)

    i = 0
    filename = "conf2022_coverpage_transparent"
    while os.path.exists('{}{:d}.png'.format(filename, i)):
        i += 1
    plt.savefig('{}{:d}.png'.format(filename, i), transparent=True, dpi=300)

    return(df)


# --------------------------------------------------------------------------
if __name__ == '__main__':
    df = main()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""usage: python status.py -h

NMDB status

Copyright (C) 2008-2023 Christian T. Steigies <steigies@physik.uni-kiel.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__author__ = "Christian T. Steigies <steigies@physik.uni-kiel.de>"
__license__ = "GPL License"


def parser(subparsers):
    # create the parser for the "down" command
    sub_p = subparsers.add_parser('status', help='status of NMDB')
    sub_p.add_argument('station', metavar='STATION', nargs='?', default='test', help='NMDB station short name')
    #sub_p.add_argument('data_type', choices=['1m', '1h', 'env', 'meta'], nargs='?', default='1m')
    #sub_p.add_argument('--dry-run', dest='dryrun', action="store_true", help='print but do not execute commands')
    #sub_p.add_argument('--file', dest='rcfile', help='config file for NMDB')
    #sub_p.add_argument('-H', '--host', dest='hostname', help='NMDB hostname to connect to')
    #sub_p.add_argument('-P', '--port', type=int, dest='port', default='3306', help='NMDB port number to connect to')
    #sub_p.add_argument('-Y', '--year', type=int, dest='year', help='year')
    #sub_p.add_argument('-M', '--month', type=int, dest='month', help='month')
    #sub_p.add_argument('-D', '--day', type=int, dest='day', help='day')
    group = sub_p.add_mutually_exclusive_group()
    group.add_argument('-v', '--verbose', action='count', default=0, help='increase verbosity level')
    group.add_argument("-q", "--quiet", action="store_true")
    sub_p.set_defaults(cmd="status")

    return(sub_p)


# --------------------------------------------------------------------------
def status(args):
    print(vars(args))

# --------------------------------------------------------------------------
if __name__ == '__main__':
    import argparse
    p = argparse.ArgumentParser(description='NMDB status interface.')
    # add subparsers!
    subparsers = p.add_subparsers() #help='nmdb sub-commands')
    parser_status = parser(subparsers)

    args = p.parse_args(args=['status', 'kiel'])
    args = p.parse_args(args=['status', 'oulu'])
    status(args)